(**************************************************************************)
(*                                                                        *)
(*  This file is part of Frama-C.                                         *)
(*                                                                        *)
(*  Copyright (C) 2007-2009                                               *)
(*    CEA (Commissariat  l'nergie Atomique)                             *)
(*                                                                        *)
(*  you can redistribute it and/or modify it under the terms of the GNU   *)
(*  Lesser General Public License as published by the Free Software       *)
(*  Foundation, version 2.1.                                              *)
(*                                                                        *)
(*  It is distributed in the hope that it will be useful,                 *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *)
(*  GNU Lesser General Public License for more details.                   *)
(*                                                                        *)
(*  See the GNU Lesser General Public License version 2.1                 *)
(*  for more details (enclosed in the file licenses/LGPLv2.1).            *)
(*                                                                        *)
(**************************************************************************)

(* [JS 2009/08/27] old cg_viewer using viewgraph. The new implementation uses
   the new dgraph toolbox embeded in ocamlgraph
module Cb = struct
  include ViewGraph.EmptyCb
  let button_one_press_on_node _env _n = () 
      (* TODO : show fct in source viewer ? *)
  let button_two_press_on_node _env _n = ()
      (* TODO : show calls in source viewer ? *)
end

module V = ViewGraph_select.VG (Cb)

let show_cg_cb _a = 
  let cg_name = Options.Filename.get () in
  let cg_name = 
    if cg_name <> "" then cg_name
    else Filename.temp_file "framaC_cg_" ".dot"
  in 
  Options.Filename.set cg_name;
  let select_init_env = ViewGraph_utils.create_graph_win "Call Graph" in
  Options.feedback "preparing file %s" cg_name;
  !Db.Syntactic_Callgraph.dump ();
  Options.feedback "file %s done." cg_name;
  try
    let env = () in
    ignore (V.open_dot_file env select_init_env cg_name)
  with ViewGraph.DotError _ ->
    GToolbox.message_box "Error"
      (Printf.sprintf "Didn't succed to build graph for %s\nSorry !" cg_name)

let main window =
  GAction.add_actions window#actions 
    [ GAction.add_action 
	"CallGraph" ~label:"_Show Call Graph" ~callback:show_cg_cb ];
  ignore
    (window#ui_manager#add_ui_from_string
       "<ui><menubar name='MenuBar'> 
              <menu action='ViewMenu'>
                 <menuitem action='CallGraph'/> 
              </menu>
           </menubar>
       </ui>")

let () = Design.register_extension main
*)

let ($) f x = f x

let scrolled_view ~packing model =
  let frame = GBin.frame ~shadow_type:`IN () in
  let aa = true (* anti-aliasing *) in
  let view = DGraphView.highlight_focus_view
    ~aa ~width:1280 ~height:1024 ~packing:frame#add model () in

  ignore $ view#set_center_scroll_region true;
  let table = GPack.table ~packing
                ~rows:2 ~columns:2 ~row_spacings:4 ~col_spacings:4 () in
  ignore $ table#attach ~left:0 ~right:1 ~top:0 ~bottom:1
           ~expand:`BOTH ~fill:`BOTH ~shrink:`BOTH ~xpadding:0 ~ypadding:0
           frame#coerce;
  let w = GRange.scrollbar `HORIZONTAL ~adjustment:view#hadjustment () in
  ignore $ table#attach ~left:0 ~right:1 ~top:1 ~bottom:2
            ~expand:`X ~fill:`BOTH ~shrink:`X ~xpadding:0 ~ypadding:0
            w#coerce;
  let w = GRange.scrollbar `VERTICAL ~adjustment:view#vadjustment () in
  ignore $ table#attach ~left:1 ~right:2 ~top:0 ~bottom:1
            ~expand:`Y ~fill:`BOTH ~shrink:`Y ~xpadding:0 ~ypadding:0
            w#coerce;

  view, table

let create_graph_win title model =
  let window = GWindow.window ~title
                 ~allow_shrink:true  ~allow_grow:true ()  in

  let vbox = GPack.vbox ~border_width:4 ~spacing:4
    ~packing:window#add () in

  let packing = vbox#pack ~expand:true ~fill:true in
  let view, _table = scrolled_view ~packing model in

(*  let hbox = GPack.hbox ~spacing:4 ~packing:vbox#pack () in*)
  window, view

let show_graph_win _a =
  let cg_name = Options.Filename.get () in
  let cg_name =
    if cg_name <> "" then cg_name
    else Filename.temp_file "framaC_cg_" ".dot"
  in
  Options.Filename.set cg_name;

  Format.printf "[call graph] preparing the file %s " cg_name;
  !Db.Syntactic_Callgraph.dump ();
  Format.printf "done.";

  try
    let model = ConcreteModel.read_dot cg_name in
    let window, view = create_graph_win "Call Graph" model in
    window#show ();
    view#adapt_zoom ()
  with ConcreteModel.DotError cmd ->
    GToolbox.message_box "Error" 
      (Printf.sprintf "%s failed\n" cmd)
    
let main window =
  GAction.add_actions window#actions 
    [ GAction.add_action 
	"CallGraph" ~label:"_Show Call Graph" ~callback:show_graph_win];
  ignore
    (window#ui_manager#add_ui_from_string
       "<ui><menubar name='MenuBar'> 
              <menu action='ViewMenu'>
                 <menuitem action='CallGraph'/> 
              </menu>
           </menubar>
       </ui>")

let () = Design.register_extension main

(*
Local Variables:
compile-command: "LC_ALL=C make -C ../.."
End:
*)

